"""
PyGoogalytics allows a user to quickly and simply download Google Analytics and Google Search Console data
in the form of a pandas dataframe.

To start, you must first create a service account and save the JSON key file locally:
https://developers.google.com/analytics/devguides/reporting/core/v4/quickstart/service-py#1_enable_the_api
Also, give the service account email address access (at least "read" level) to the Search Console or Google Analytics
accounts you want to access. Now you can use the PyGoogalytics API wrapper.

Follow the implementation example below:

```
from pygoogalytics.client import Client
googalytics_client = Client(key_file_path='<path-to-your-key-file>')
g_wrapper = googalytics_client.wrapper(
  sc_domain='<search-console-domain>',
  view_id='<ga3-view-id>',
  ga4_property_id='<ga4-property-id>'
)
```
"""

__version__ = "0.1.0"
__author__ = 'Joshua Prettyman'
__credits__ = 'Blink SEO'

import logging
pga_logger = logging.getLogger(__name__)



import json

from google.oauth2 import service_account  # pip install --upgrade google-auth
from googleapiclient import discovery  # pip install --upgrade google-api-python-client
from google.analytics.data_v1beta import BetaAnalyticsDataClient  # pip install google-analytics-data


def resources_from_key_file(path: str):
    with open(path, 'rb') as _file:
        _json_string = _file.read().decode('utf8')
    return resources_from_json(json_string=_json_string)


def resources_from_json(json_string: str):
    _secrets = json.loads(json_string)
    _project_credentials = get_analytics_project_credentials(secrets=_secrets)
    return get_analytics_resources(analytics_project_credentials=_project_credentials)


def get_analytics_project_credentials(secrets: dict):
    analytics_project_scopes = ['https://www.googleapis.com/auth/analytics.readonly',
                                'https://www.googleapis.com/auth/webmasters.readonly',
                                'https://www.googleapis.com/auth/webmasters',
                                'https://www.googleapis.com/auth/cloud-platform']

    project_credentials = service_account.Credentials.from_service_account_info(
        info=secrets).with_scopes(scopes=analytics_project_scopes)
    return project_credentials


def get_analytics_resources(analytics_project_credentials):
    ga3_resource = discovery.build('analyticsreporting', 'v4', credentials=analytics_project_credentials)
    gsc_resource = discovery.build('searchconsole', 'v1', credentials=analytics_project_credentials)
    ga4_data_client = BetaAnalyticsDataClient(credentials=analytics_project_credentials)
    return ga3_resource, ga4_data_client, gsc_resource



import datetime
import re
from typing import List, Tuple, Optional


QUESTION = r"((^what)|(^why)|(^how)|(^can )|(^do )|(^does)|(^where)|(^who(se)? )|(who'?s )|(^which)|(^when)|(^is )|(^are )|(.*\?$))"
TRANSACTION = r"(.*((buy)|(cost)|(price)|(cheap)|(pricing)|(affordable)))"
INVESTIGATION = r"(.*((best)|(most)|(cheapest)|( vs)|( v\.s\.)))"
URL = r"https?://(www\.)?[\w\-_+]*(\.\w{2,4}){0,2}/"

RE_QUESTION = re.compile(QUESTION)
RE_TRANSACTIONAL = re.compile(TRANSACTION)
RE_INVESTIGATION = re.compile(INVESTIGATION)
RE_URL = re.compile(URL)
RE_C2S = re.compile(r"(?<!^)(?=[A-Z])")


def strip_url(url: str) -> str:
    url = url.split('?')[0]
    _match = RE_URL.match(url)
    if _match:
        return url[_match.span()[1] - 1:]
    else:
        return url


def url_extract_parameter(url: str) -> Optional[str]:
    if '?' in url:
        return url.split('?')[1]
    else:
        return None


def url_strip_domain(url: str) -> Optional[str]:
    _match = RE_URL.match(url)
    if _match:
        return url[_match.span()[1] - 1:]
    else:
        return url


def date_range_string(dates: List[datetime.date],
                      alternate_text: str = ""):
    if len(dates) == 0:
        date_range_str = alternate_text
    elif len(dates) == 1:
        date_range_str = f"{dates[0].strftime('%Y-%m-%d')}"
    else:
        _min, _max, _days, _range = date_range(dates)
        date_range_str = f"{_min.strftime('%Y-%m-%d')} to " \
                         f"{_max.strftime('%Y-%m-%d')} ({100 * _range:.0f}%)"
    return date_range_str


def date_range(dates: List[datetime.date]) -> Tuple[datetime.date, datetime.date, int, float]:
    _max = max(dates)
    _min = min(dates)
    _days = (_max - _min).days
    _coverage = len(set(dates))/_days
    return _min, _max, _days, _coverage


def test_time(t: datetime.datetime, seconds: float) -> Optional[bool]:
    if not isinstance(t, datetime.datetime):
        return None
    _diff = (datetime.datetime.now() - t).seconds
    if _diff < seconds:
        return True
    else:
        return False



from .resource_utils import resources_from_json, resources_from_key_file
from .wrapper import GoogalyticsWrapper
from . import pga_logger


class Client:
    """
    The Client class holds the credentials for a project
    which can then be used to create a GoogalyticsWrapper object.

    example implementation:
    from pygoogalytics.client import Client
    googalytics_client = Client(key_file_path='<path-to-your-key-file>')
    """
    def __init__(self,
                 key_file_path: str = None,
                 key_file_json: str = None
                 ):

        self.gsc_resource = None
        self.ga3_resource = None
        self.ga4_resource = None

        if key_file_json:
            self._build_from_json(json_string=key_file_json)
            pga_logger.info(f"initialised PyGoogalytics Client object from key file")
        elif key_file_path:
            self._build_from_key_file(path=key_file_path)
            pga_logger.info(f"initialised PyGoogalytics Client object from JSON string")

    def _build_from_json(self, json_string: str):
        _ga3_resource, _ga4_resource, _gsc_resource = resources_from_json(json_string=json_string)
        self.gsc_resource = _gsc_resource
        self.ga3_resource = _ga3_resource
        self.ga4_resource = _ga4_resource

    def _build_from_key_file(self, path: str):
        _ga3_resource, _ga4_resource, _gsc_resource = resources_from_key_file(path=path)
        self.gsc_resource = _gsc_resource
        self.ga3_resource = _ga3_resource
        self.ga4_resource = _ga4_resource

    def wrapper(self,
                sc_domain: str = None,
                view_id: str = None,
                ga4_property_id: str = None,
                ) -> GoogalyticsWrapper:
        """
        The GoogalyticsWrapper requires different arguments to access data depending on the source.
        @param sc_domain: required for GSC data. This is the url-like string you see in the Google Search Console web application when selecting the site. It is either a full url (e.g. `https://www.example.com/`) or something like `sc_domain:example.com`
        @param view_id: required for GA3 data: the "view_id" you see in "settings" on the GA web application. This is usually an 8- or 9-digit number, passed as a string
        @param ga4_property_id: required for GA4 data. Note that GA4 is currently not fully supported by PyGoogalytics
        @returns: GoogalyticsWrapper object.
        """
        return GoogalyticsWrapper(
            gsc_resource=self.gsc_resource,
            ga3_resource=self.ga3_resource,
            ga4_resource=self.ga4_resource,
            sc_domain=sc_domain,
            view_id=view_id,
            ga4_property_id=ga4_property_id
        )

    def __bool__(self):
        if self.gsc_resource is not None or self.ga3_resource is not None or self.ga4_resource is not None:
            return True
        else:
            return False

    def __dict__(self):
        return {
            'gsc_resource': self.gsc_resource.__repr__(),
            'ga3_resource': self.ga3_resource.__repr__(),
            'ga4_resource': self.ga4_resource.__repr__()
        }

    def __repr__(self):
        _s = 'PyGoogalytics Client object:\n'
        for _k, _v in self.__dict__().items():
            _s += " - {_k}: {_v}"
        return _s


from setuptools import setup

setup(
    name='pygoogalytics',
    version='0.1.0',
    description='',
    url='https://github.com/Blink-SEO/pygoogalytics',
    author='Joshua Prettyman',
    author_email='joshua@blinkseo.co.uk',
    license='MIT',
    packages=['pygoogalytics'],
    install_requires=[
        'pandas',
        'google-api-python-client',
        'google-analytics-data'
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)

# python setup.py check
# python setup.py sdist
# twine upload dist/*
